@@ -6,17 +6,17 @@
 # Usage:
 # Export tagged, public versions
 
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --bld --cln nco-2_2_0 # Build, do not install
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --nst_all nco-2_2_0 # Install, do not build
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --bld --cln --nst_all nco-2_2_0 # Build and install
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --acd_cnt nco-2_2_0
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --acd_prs nco-2_2_0
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --cgd_cnt nco-2_2_0
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --cray_prs nco-2_2_0
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --bbl_cnt nco-2_2_0
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --blk_cnt nco-2_2_0
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --dat_cnt nco-2_2_0
-# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --ute_prs nco-2_2_0
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --bld --cln nco-2_7_1 # Build, do not install
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --nst_all nco-2_7_1 # Install, do not build
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --bld --cln --nst_all nco-2_7_1 # Build and install
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --acd_cnt nco-2_7_1
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --acd_prs nco-2_7_1
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --cgd_cnt nco-2_7_1
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --cray_prs nco-2_7_1
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --bbl_cnt nco-2_7_1
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --blk_cnt nco-2_7_1
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --dat_cnt nco-2_7_1
+# ${HOME}/nco/bld/nco_dst.pl --dbg=2 --cln --ute_prs nco-2_7_1
 
 # Export daily snapshot
 # ${HOME}/nco/bld/nco_dst.pl --dbg=2 
@@ -24,13 +24,13 @@
 
 # Machines requiring interactive builds
 # cd ${HOME}/nco;cvs update;cd bld;make;make tst
-# scp ${HOME}/nco/bld/nco_dst.pl goldhill.cgd.ucar.edu:/home/zender/nco/bld/nco_dst.pl
+# scp ${HOME}/nco/bld/nco_dst.pl dust.ps.uci.edu:/home/zender/nco/bld/nco_dst.pl
 
 BEGIN{
     unshift @INC,$ENV{'HOME'}.'/perl'; # Location of csz.pl and DBG.pm HaS98 p. 170
 } # end BEGIN
 
-my $CVS_Header='$Header: /cvsroot/nco/nco/bld/nco_dst.pl,v 1.79 2002/02/05 08:15:06 zender Exp $';
+my $CVS_Header='$Header: /cvsroot/nco/nco/bld/nco_dst.pl,v 1.97 2003/01/09 01:17:57 zender Exp $';
 
 # Specify modules
 use strict; # Protect all namespaces
@@ -66,9 +66,9 @@ my ($rsh_cmd,$rcp_cmd,$cp_cmd,$rm_cmd,$mkdir_cmd,$cvs_cmd);
 my $False=0;
 my $True=1;
 
-my $CVS_Date='$Date: 2002/02/05 08:15:06 $';
-my $CVS_Id='$Id: nco_dst.pl,v 1.79 2002/02/05 08:15:06 zender Exp $';
-my $CVS_Revision='$Revision: 1.79 $';
+my $CVS_Date='$Date: 2003/01/09 01:17:57 $';
+my $CVS_Id='$Id: nco_dst.pl,v 1.97 2003/01/09 01:17:57 zender Exp $';
+my $CVS_Revision='$Revision: 1.97 $';
 my $CVSROOT='zender@cvs.nco.sourceforge.net:/cvsroot/nco'; # CVS repository
 my $HOME=$ENV{'HOME'};
 my $HOST=$ENV{'HOST'};
@@ -77,20 +77,21 @@ my $bld=$False; # Option bld; Whether to rebuild netCDF distribution
 my $cp_cmd='cp -p -f'; # Command that behaves like cp
 my $cvs_cmd='cvs'; # Command that behaves like cvs (Use cvs -t for verbose output)
 my $data_nm=$ENV{'DATA'};
-my $ftp_drc='/ftp/pub/zender/nco'; # Directory on FTP machine where repository resides
+my $ftp_drc='/var/ftp/pub/zender/nco'; # Directory on FTP machine where repository resides
 my $ftp_drc_mrr='/home/groups/ftp/pub/nco'; # Directory on FTP mirror machine where repository resides
-my $ftp_mch='ftp.cgd.ucar.edu'; # Machine where FTP repository resides
+my $ftp_mch='dust.ps.uci.edu'; # Machine where FTP repository resides
 my $ftp_mch_mrr='nco.sourceforge.net'; # Machine where FTP repository mirror resides
 my $main_trunk_tag='nco';
 my $mkdir_cmd='mkdir -p'; # Command that behaves like mkdir
 my $mdl_sng='nco'; # Base of module name
 my $rm_cmd='rm -f'; # Command that behaves like rm
 my $rcp_cmd='scp -p'; # Command that behaves like rcp
+my $rcp_cmd_no_prs_prm='scp'; # Command that behaves like rcp and does not try to preserve permissions
 my $rsh_cmd='ssh'; # Command that behaves like rsh
 my $usr_nm=$ENV{'USER'};
 my $vrs_tag='';
-my $www_mch='ftp.cgd.ucar.edu'; # WWW machine for package
-my $www_drc='/web/web-data/cms/nco'; # WWW directory for package
+my $www_mch='dust.ps.uci.edu'; # WWW machine for package
+my $www_drc='/var/www/html/nco'; # WWW directory for package
 my $www_mch_mrr='nco.sourceforge.net'; # WWW machine for package mirror
 my $www_drc_mrr='/home/groups/n/nc/nco/htdocs'; # WWW directory for package mirror
 
@@ -103,6 +104,7 @@ my $acd_cnt=$False; # Option acd_cnt; Install version in acd contrib
 my $acd_prs=$False; # Option acd_prs; Install version in acd personal
 my $bbl_cnt=$False; # Option bbl_cnt; Install version in babyblue contrib
 my $blk_cnt=$False; # Option blk_cnt; Install version in blackforest contrib
+my $bls_cnt=$False; # Option bls_cnt; Install version in bluesky contrib
 my $dat_cnt=$False; # Option dat_cnt; Install version in dataproc contrib
 my $ute_prs=$False; # Option ute_prs; Install version in ute personal
 my $cgd_cnt=$False; # Option cgd_cnt; Install version in CGD contrib
@@ -124,14 +126,7 @@ if($data_nm eq ''){$data_nm='/data/'.$usr_nm;}
 my $dst_pth_pfx=$data_nm; # Parent of build directory
 if($dst_pth_pfx eq $HOME){die "$prg_nm: ERROR \$dst_pth_pfx eq $dst_pth_pfx";} # This could be disastrous
 if($rm_cmd =~ m/( -r)|( -R)|( --recursive)/){die "$prg_nm: ERROR Dangerous setting \$rm_cmd eq $rm_cmd";} # This would be disastrous
-#  if($HOST =~ m/cgd\.ucar\.edu/ || $HOST =~ m/sanitas/ || $HOST =~ m/goldhill/ || $HOST =~ m/dataproc/){
-#  # CVS 1.10 has a bug where 'cvs -d :ext:user@host:repositorypath export -kkv -r revision_tag module' fails
-#  # Workaround: Export from a machine cross-mounted to /fs/cgd so that :ext:user@host is not necessary
-#      $CVSROOT='/home/zender/cvs'; # CVS repository
-#  }else{
-#      $CVSROOT=':ext:'.$usr_nm.'@goldhill.cgd.ucar.edu:/home/zender/cvs'; # CVS repository
 #      $CVSROOT=':pserver:anonymous@cvs.nco.sourceforge.net:/cvsroot/nco'; # CVS repository
-#  } # endif CVSROOT
 
 $prg_dsc='NCO distribution maker'; # Program description
 ($prg_nm,$prg_vrs)=$CVS_Id =~ /: (.+).pl,v ([\d.]+)/; # Program name and version
@@ -237,9 +232,9 @@ if($bld){
     }else{
 	cmd_prc("$cvs_cmd -d $CVSROOT export -kkv -r $vrs_tag -d $dst_vrs nco"); # Export
     } # endelse
-    cmd_prc("printf $dst_vrs > $dst_pth_bld/doc/VERSION"); # Stamp version in VERSION file in exported files
-    cmd_prc("printf $dst_vrs > $HOME/nco/doc/VERSION"); # Stamp version in VERSION file in development directory
-#    cmd_prc("ln -s $dst_pth_bld/bld/nco.spec $dst_pth_bld/bld/nco-$dst_vrs.spec"); # Stamp version in VERSION file
+    cmd_prc("printf $nco_vrs > $dst_pth_bld/doc/VERSION"); # Stamp version in VERSION file in exported files
+    cmd_prc("printf $nco_vrs > $HOME/nco/doc/VERSION"); # Stamp version in VERSION file in development directory
+#    cmd_prc("ln -s $dst_pth_bld/bld/nco.spec $dst_pth_bld/bld/nco-$nco_vrs.spec"); # Stamp version in VERSION file
     
 # Make sure documentation files are up to date
     my $bld_pth=$dst_pth_pfx.'/'."$dst_vrs".'/bld';
@@ -269,17 +264,15 @@ if($bld){
     } # endif 
     
 # Update SourceForge mirror
+# Use rcp_cmd_no_prs_prm because scp appears to quit without notice if too many warnings errors occur copying first files
+# Usually scp_cmd includes -p switch to preserve permissions and times, but sourceforge server does not allow this
+# Hence it generates warnings when used with -p, and is not properly updated
     if($True){
-	cmd_prc("$rcp_cmd $dst_fl $ftp_mch_mrr:$ftp_drc_mrr"); # Copy local tarfile to FTP server
+	cmd_prc("$rcp_cmd_no_prs_prm $dst_fl $ftp_mch_mrr:$ftp_drc_mrr"); # Copy local tarfile to FTP server
 	cmd_prc("$rsh_cmd $ftp_mch_mrr $rm_cmd $ftp_drc_mrr/nco.tar.gz");
 	cmd_prc("$rsh_cmd $ftp_mch_mrr \"cd $ftp_drc_mrr; ln -s $dst_fl nco.tar.gz\"");
-	cmd_prc("$rcp_cmd $dst_pth_bld/doc/index.shtml $www_mch_mrr:$www_drc_mrr");
-	cmd_prc("$rcp_cmd $dst_pth_bld/doc/nco_news.shtml $www_mch_mrr:$www_drc_mrr");
-	cmd_prc("$rcp_cmd $dst_pth_bld/doc/nco.html $dst_pth_bld/doc/nco.info* $dst_pth_bld/doc/nco.dvi $dst_pth_bld/doc/nco.pdf $dst_pth_bld/doc/nco.ps $dst_pth_bld/doc/nco.ps $dst_pth_bld/doc/nco.texi $www_mch_mrr:$www_drc_mrr");
-	cmd_prc("$rcp_cmd $dst_pth_bld/doc/README $www_mch_mrr:$www_drc_mrr");
-	cmd_prc("$rcp_cmd $dst_pth_bld/doc/TODO $www_mch_mrr:$www_drc_mrr");
-	cmd_prc("$rcp_cmd $dst_pth_bld/doc/VERSION $www_mch_mrr:$www_drc_mrr");
-	cmd_prc("$rcp_cmd $dst_pth_bld/doc/ChangeLog $www_mch_mrr:$www_drc_mrr");
+	cmd_prc("$rcp_cmd_no_prs_prm $dst_pth_bld/doc/index.shtml $dst_pth_bld/doc/nco_news.shtml $dst_pth_bld/doc/README $dst_pth_bld/doc/TODO $dst_pth_bld/doc/VERSION $dst_pth_bld/doc/ChangeLog $www_mch_mrr:$www_drc_mrr");
+	cmd_prc("$rcp_cmd_no_prs_prm $dst_pth_bld/doc/nco.html $dst_pth_bld/doc/nco.info* $dst_pth_bld/doc/nco.dvi $dst_pth_bld/doc/nco.pdf $dst_pth_bld/doc/nco.ps $dst_pth_bld/doc/nco.ps $dst_pth_bld/doc/nco.texi $www_mch_mrr:$www_drc_mrr");
     } # endif SourceForge
 
 # Housekeeping
