@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/ncatted.c,v 1.39 2002/01/28 10:06:53 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/ncatted.c,v 1.57 2003/01/09 00:27:37 rorik Exp $ */
 
 /* ncatted -- netCDF attribute editor */
 
 /* Purpose: Add, create, delete, or overwrite attributes in a netCDF file */
 
-/* Copyright (C) 1995--2002 Charlie Zender
+/* Copyright (C) 1995--2003 Charlie Zender
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -63,6 +63,9 @@
    Delete attribute:
    ncatted -D 5 -O -a float_att,att_var,d,,, in.nc foo.nc
 
+   Delete all attributes for given var:
+   ncatted -D 5 -O -a ,att_var,d,,, in.nc foo.nc
+
    Modify existing float:
    ncatted -D 5 -O -a float_att,att_var,m,f,74 in.nc foo.nc
 
@@ -96,9 +99,11 @@
    ncatted -D 5 -O -a float_att,,m,f,74 in.nc foo.nc
 
    Verify results:
-   ncks -C -h foo.nc | m
+   ncks -C -h foo.nc | m */
 
-   */
+#ifdef HAVE_CONFIG_H
+#include <config.h> /* Autotools tokens */
+#endif /* !HAVE_CONFIG_H */
 
 /* Standard C headers */
 #include <math.h> /* sin cos cos sin 3.14159 */
@@ -108,9 +113,11 @@
 #include <sys/stat.h> /* stat() */
 #include <time.h> /* machine time */
 #include <unistd.h> /* all sorts of POSIX stuff */
-/* #include <assert.h> */ /* assert() debugging macro */
-/* #include <errno.h> */ /* errno */
-/* #include <malloc.h> */ /* malloc() stuff */
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>  /* getopt_long() */
+#else
+#include "nco_getopt.h"
+#endif /* !HAVE_GETOPT_H */
 
 /* 3rd party vendors */
 #include <netcdf.h> /* netCDF definitions */
@@ -119,7 +126,7 @@
 /* #define MAIN_PROGRAM_FILE MUST precede #include nco.h */
 #define MAIN_PROGRAM_FILE
 #include "nco.h" /* NCO definitions */
-
+#include "libnco.h" /* netCDF operator library */
 int 
 main(int argc,char **argv)
 {
@@ -142,8 +149,8 @@ main(int argc,char **argv)
   char *fl_pth=NULL; /* Option p */
   char *time_bfr_srt;
   char *cmd_ln;
-  char CVS_Id[]="$Id: ncatted.c,v 1.39 2002/01/28 10:06:53 zender Exp $"; 
-  char CVS_Revision[]="$Revision: 1.39 $";
+  char CVS_Id[]="$Id: ncatted.c,v 1.57 2003/01/09 00:27:37 rorik Exp $"; 
+  char CVS_Revision[]="$Revision: 1.57 $";
   
   aed_sct *aed_lst=NULL_CEWI;
 
@@ -162,8 +169,27 @@ main(int argc,char **argv)
 
   time_t clock;
 
-  /* Start the clock and save the command line */ 
-  cmd_ln=cmd_ln_sng(argc,argv);
+  static struct option opt_lng[]=
+    {
+      {"append",no_argument,0,'A'},
+      {"attribute",required_argument,0,'a'},
+      {"debug",required_argument,0,'D'},
+      {"dbg_lvl",required_argument,0,'D'},
+      {"history",no_argument,0,'h'},
+      {"hst",no_argument,0,'h'},
+      {"local",required_argument,0,'l'},
+      {"overwrite",no_argument,0,'O'},
+      {"path",required_argument,0,'p'},
+      {"retain",no_argument,0,'R'},
+      {"version",no_argument,0,'r'},
+      {"vrs",no_argument,0,'r'},
+      {"help",no_argument,0,'?'},
+      {0,0,0,0}
+    }; /* end opt_lng */
+  int opt_idx=0; /* Index of current long option into opt_lng array */
+
+  /* Start clock and save command line */ 
+  cmd_ln=nco_cmd_ln_sng(argc,argv);
   clock=time((time_t *)NULL);
   time_bfr_srt=ctime(&clock); time_bfr_srt=time_bfr_srt; /* Avoid compiler warning until variable is used for something */
   
@@ -171,8 +197,8 @@ main(int argc,char **argv)
   prg_nm=prg_prs(argv[0],&prg);
 
   /* Parse command line arguments */
-  opt_sng="Aa:D:hl:Op:Rr";
-  while((opt = getopt(argc,argv,opt_sng)) != EOF){
+  opt_sng="Aa:D:hl:Op:Rr-:";
+  while((opt = getopt_long(argc,argv,opt_sng,opt_lng,&opt_idx)) != EOF){
     switch(opt){
     case 'A': /* Toggle FORCE_APPEND */
       FORCE_APPEND=!FORCE_APPEND;
@@ -202,33 +228,41 @@ main(int argc,char **argv)
     case 'r': /* Print CVS program information and copyright notice */
       (void)copyright_prn(CVS_Id,CVS_Revision);
       (void)nco_lib_vrs_prn();
-      exit(EXIT_SUCCESS);
+      nco_exit(EXIT_SUCCESS);
+      break;
+    case '?': /* Print proper usage */
+      (void)nco_usg_prn();
+      nco_exit(EXIT_FAILURE);
+      break;
+    case '-': /* Long options are not allowed */
+      (void)fprintf(stderr,"%s: ERROR Long options are not available in this build. Use single letter options instead.\n",prg_nm_get());
+      nco_exit(EXIT_FAILURE);
       break;
     default: /* Print proper usage */
-      (void)usg_prn();
-      exit(EXIT_FAILURE);
+      (void)nco_usg_prn();
+      nco_exit(EXIT_FAILURE);
     } /* end switch */
   } /* end while loop */
   
   /* Process positional arguments and fill in filenames */
-  fl_lst_in=fl_lst_mk(argv,argc,optind,&nbr_fl,&fl_out);
+  fl_lst_in=nco_fl_lst_mk(argv,argc,optind,&nbr_fl,&fl_out);
   if(fl_out != NULL) OUTPUT_TO_NEW_NETCDF_FILE=True; else fl_out=fl_lst_in[0];
 
   if(nbr_aed == 0){
     (void)fprintf(stdout,"%s: ERROR must specify an attribute to edit\n",prg_nm);
-    usg_prn();
-    exit(EXIT_FAILURE);
+    nco_usg_prn();
+    nco_exit(EXIT_FAILURE);
   } /* end if */ 
 
-  /* Make a uniform list of the user-specified rename structures */
+  /* Make uniform list of user-specified attribute edit structures */
   if(nbr_aed > 0) aed_lst=prs_aed_lst(nbr_aed,aed_arg);
 
-  /* We now have the final list of attributes to edit. */
+  /* We now have final list of attributes to edit */
   
   /* Parse filename */
-  fl_in=fl_nm_prs(fl_in,0,&nbr_fl,fl_lst_in,nbr_abb_arg,fl_lst_abb,fl_pth);
+  fl_in=nco_fl_nm_prs(fl_in,0,&nbr_fl,fl_lst_in,nbr_abb_arg,fl_lst_abb,fl_pth);
   /* Make sure file is on local system and is readable or die trying */
-  fl_in=fl_mk_lcl(fl_in,fl_pth_lcl,&FILE_RETRIEVED_FROM_REMOTE_LOCATION);
+  fl_in=nco_fl_mk_lcl(fl_in,fl_pth_lcl,&FILE_RETRIEVED_FROM_REMOTE_LOCATION);
 
   if(OUTPUT_TO_NEW_NETCDF_FILE){
 
@@ -251,14 +285,14 @@ main(int argc,char **argv)
         } /* end while */
         
         if(usr_reply == 'n'){
-          exit(EXIT_SUCCESS);
+          nco_exit(EXIT_SUCCESS);
         } /* end if */
       } /* end if */
     } /* end if */
     
     /* Copy input file to output file, then search through output, editing attributes along the way.
        This avoids possible XDR translation performance penalty of copying each variable with netCDF. */
-    (void)fl_cp(fl_in,fl_out);
+    (void)nco_fl_cp(fl_in,fl_out);
 
   } /* end if */
 
@@ -278,20 +312,20 @@ main(int argc,char **argv)
       if(!strcmp(aed_lst[idx].var_nm,"global")) aed_lst[idx].id=NC_GLOBAL; else (void)nco_inq_varid(nc_id,aed_lst[idx].var_nm,&aed_lst[idx].id);
 
       /* Edit attribute */
-      (void)aed_prc(nc_id,aed_lst[idx].id,aed_lst[idx]);
+      (void)nco_aed_prc(nc_id,aed_lst[idx].id,aed_lst[idx]);
 
     }else{ /* var_nm == NULL */
       /* Perform operation for every variable for which it makes sense */
       
       /* Edit attribute for every variable */
-      for(idx_var=0;idx_var<nbr_var_fl;idx_var++) (void)aed_prc(nc_id,idx_var,aed_lst[idx]);
+      for(idx_var=0;idx_var<nbr_var_fl;idx_var++) (void)nco_aed_prc(nc_id,idx_var,aed_lst[idx]);
 
     } /* end else var_nm == NULL */
 
   } /* end loop over idx */
   
   /* Catenate the timestamped command line to the "history" global attribute */
-  if(HISTORY_APPEND) (void)hst_att_cat(nc_id,cmd_ln);
+  if(HISTORY_APPEND) (void)nco_hst_att_cat(nc_id,cmd_ln);
   
   /* Take file out of define mode */
   (void)nco_enddef(nc_id);
@@ -300,23 +334,21 @@ main(int argc,char **argv)
   nco_close(nc_id);
   
   /* Remove local copy of file */
-  if(FILE_RETRIEVED_FROM_REMOTE_LOCATION && REMOVE_REMOTE_FILES_AFTER_PROCESSING) (void)fl_rm(fl_in);
+  if(FILE_RETRIEVED_FROM_REMOTE_LOCATION && REMOVE_REMOTE_FILES_AFTER_PROCESSING) (void)nco_fl_rm(fl_in);
 
   if(rcd != NC_NOERR) nco_err_exit(rcd,"main");
-  Exit_gracefully();
+  nco_exit_gracefully();
   return EXIT_SUCCESS;
 } /* end main() */
 
-aed_sct *
-prs_aed_lst(int nbr_aed,char **aed_arg)
-/* 
-   int nbr_aed: input number of attributes
-   char **aed_arg: input list of user-specified dimension attributes
-   aed_sct *prs_aed_lst(): output structure holding user-specified strings for attributes
- */
+aed_sct * /* O [sct] List of attribute edit structures */
+prs_aed_lst /* [fnc] Parse user-specified attribute edits into structure list */
+(int nbr_aed, /* I [nbr] Number of attributes in list */
+ char **aed_arg) /* I [sng] List of user-specified attribute edits */
 {
-  /* Routine to set name, type, size, and value elements of a comma separated list of attribute info. 
-     This routine merely evaluates syntax of input expressions but does not validate attributes or variables against those present in input netCDF file. */
+  /* Purpose: Parse name, type, size, and value elements of comma-separated list of attribute edit information
+     Routine merely evaluates syntax of input expressions
+     Routine does not validate attributes or variables against those present in input netCDF file */
 
   /* Options are:
      -a att_nm,var_nm,mode,att_typ,att_val (modifies attribute att_nm for the single variable var_nm)
@@ -348,10 +380,10 @@ prs_aed_lst(int nbr_aed,char **aed_arg)
 
      -o: Attribute overwrite mode
      Write attribute att_nm with value att_val to variable var_nm, overwriting existing attribute att_nm, if any.
-     This is the default mode.
+     This is default mode.
    */
 
-  void usg_prn(void);
+  void nco_usg_prn(void);
 
   char **arg_lst;
 
@@ -374,21 +406,20 @@ prs_aed_lst(int nbr_aed,char **aed_arg)
     /* Check syntax */
     if(
        arg_nbr < 5 || /* Need more info */
-       arg_lst[0] == NULL || /*  att_nm not specified */
-       arg_lst[2] == NULL || /*  mode not specified */
+       /* arg_lst[0] == NULL || */ /* att_nm not specified */
+       arg_lst[2] == NULL || /* mode not specified */
        (*(arg_lst[2]) != 'd' && (arg_lst[3] == NULL || (arg_lst[idx_att_val_arg] == NULL && *(arg_lst[3]) != 'c'))) || /* att_typ and att_val must be specified when mode is not delete, except that att_val = "" is valid for character type */
        False){
       (void)fprintf(stdout,"%s: ERROR in attribute edit specification %s\n",prg_nm_get(),aed_arg[idx]);
-      exit(EXIT_FAILURE);
+      nco_exit(EXIT_FAILURE);
     } /* end if */
 
     /* Initialize structure */
-    /* aed strings which are not explicitly set by the user will remain as NULLs,
-       i.e., specifying the default setting will appear as if nothing at all was set.
-       Hopefully, in the routines that follow, the branch followed by an aed for which
-       all the default settings were specified (e.g.,"-a foo,,,,") will yield the same result
-       as the branch for which all defaults were set.
-     */
+    /* aed strings not explicitly set by user remain NULL,
+       i.e., specifying default setting appears as if nothing was set.
+       Hopefully, in routines that follow, the branch followed by an aed for which
+       all default settings were specified (e.g.,"-a foo,,,,") will yield same result
+       as branch for which all defaults were set. */
     aed_lst[idx].att_nm=NULL;
     aed_lst[idx].var_nm=NULL;
     aed_lst[idx].val.vp=NULL;
@@ -401,7 +432,7 @@ prs_aed_lst(int nbr_aed,char **aed_arg)
     aed_lst[idx].att_nm=arg_lst[0];
     aed_lst[idx].var_nm=arg_lst[1];
 
-    /* fxm: These switches should be changed to string comparisons someday */
+    /* fxm: Change these switches to string comparisons someday */
     /* Set mode of current aed structure */
     /* Convert single letter code to mode enum */
     /*    if(!strstr("append",arg_lst[2])){aed_lst[idx].mode=aed_append;
@@ -410,15 +441,15 @@ prs_aed_lst(int nbr_aed,char **aed_arg)
     }else if(!strstr("modify",arg_lst[2])){aed_lst[idx].mode=aed_modify;
     }else if(!strstr("overwrite",arg_lst[2])){aed_lst[idx].mode=aed_overwrite;} */
     switch(*(arg_lst[2])){
-    case 'a':	aed_lst[idx].mode=aed_append; break;
-    case 'c':	aed_lst[idx].mode=aed_create; break;
-    case 'd':	aed_lst[idx].mode=aed_delete; break;
-    case 'm':	aed_lst[idx].mode=aed_modify; break;
-    case 'o':	aed_lst[idx].mode=aed_overwrite; break;
+    case 'a': aed_lst[idx].mode=aed_append; break;
+    case 'c': aed_lst[idx].mode=aed_create; break;
+    case 'd': aed_lst[idx].mode=aed_delete; break;
+    case 'm': aed_lst[idx].mode=aed_modify; break;
+    case 'o': aed_lst[idx].mode=aed_overwrite; break;
     default: 
       (void)fprintf(stderr,"%s: ERROR `%s' is not a supported mode\n",prg_nm_get(),arg_lst[2]);
       (void)fprintf(stderr,"%s: HINT: Valid modes are `a' = append, `c' = create,`d' = delete, `m' = modify, and `o' = overwrite",prg_nm_get());
-      exit(EXIT_FAILURE);
+      nco_exit(EXIT_FAILURE);
       break;
     } /* end switch */
 
@@ -438,7 +469,7 @@ prs_aed_lst(int nbr_aed,char **aed_arg)
       default: 
 	(void)fprintf(stderr,"%s: ERROR `%s' is not a supported netCDF data type\n",prg_nm_get(),arg_lst[3]);
 	(void)fprintf(stderr,"%s: HINT: Valid data types are `c' = char, `f' = float, `d' = double,`s' = short, `l' = long, `b' = byte",prg_nm_get());
-	exit(EXIT_FAILURE);
+	nco_exit(EXIT_FAILURE);
 	break;
       } /* end switch */
       
@@ -489,11 +520,11 @@ prs_aed_lst(int nbr_aed,char **aed_arg)
 	case NC_SHORT: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.sp[lmn]=(short)val_arg_dbl[lmn];} break; 
 	case NC_CHAR: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.cp[lmn]=(unsigned char)val_arg_dbl[lmn];} break; 
 	case NC_BYTE: for(lmn=0L;lmn<aed_lst[idx].sz;lmn++) {aed_lst[idx].val.bp[lmn]=(signed char)val_arg_dbl[lmn];} break; 
-	default: nco_dfl_case_nctype_err(); break;
+	default: nco_dfl_case_nc_type_err(); break;
 	} /* end switch */
 	
 	/* Free array used to hold double values */
-	val_arg_dbl=nco_free(val_arg_dbl);
+	val_arg_dbl=(double *)nco_free(val_arg_dbl);
       } /* end else */
       /* Un-typecast pointer to values after access */
       (void)cast_nctype_void(aed_lst[idx].type,&aed_lst[idx].val);
