@@ -1,43 +1,11 @@
-/* $Header: /cvsroot/nco/nco/src/nco/nco_netcdf.c,v 1.17 2002/01/23 09:24:14 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/nco_netcdf.c,v 1.28 2002/12/30 02:56:15 zender Exp $ */
 
 /* Purpose: Wrappers for netCDF 3.X C-library */
 
-/* Copyright (C) 1995--2002 Charlie Zender
-   
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2
-   of the License, or (at your option) any later version.
-   
-   As a special exception to the terms of the GPL, you are permitted 
-   to link the NCO source code with the NetCDF and HDF libraries 
-   and distribute the resulting executables under the terms of the GPL, 
-   but in addition obeying the extra stipulations of the netCDF and 
-   HDF library licenses.
-   
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-   
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-   
-   The file LICENSE contains the GNU General Public License, version 2
-   It may be viewed interactively by typing, e.g., ncks -L
-   
-   The author of this software, Charlie Zender, would like to receive
-   your suggestions, improvements, bug-reports, and patches for NCO.
-   Please contact the project at http://sourceforge.net/projects/nco or by writing
-   
-   Charlie Zender
-   Department of Earth System Science
-   University of California at Irvine
-   Irvine, CA 92697-3100
-*/
+/* Copyright (C) 1995--2003 Charlie Zender
+   This software is distributed under the terms of the GNU General Public License
+   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
-/* Personal headers */
 #include "nco_netcdf.h" /* Wrappers for netCDF 3.X C-library */
 
 /* Organization: 
@@ -67,11 +35,11 @@
 
 /* Utility routines not defined by netCDF library, but useful in working with it */
 void 
-nco_err_exit
+nco_err_exit /* [fnc] Print netCDF error message, routine name, then exit */
 (int rcd, /* I [enm] netCDF error code */ 
  char *msg) /* I [sng] supplemental error message */
 {
-  /* Purpose: Print netCDF error message, routine name, and exit 
+  /* Purpose: Print netCDF error message, routine name, then exit
      Routine is called by all wrappers when a fatal error is encountered
      msg variable allows wrapper to pass more descriptive information than 
      is contained in the netCDF-defined error message.
@@ -83,8 +51,9 @@ nco_err_exit
   } /* endif error */
 } /* end nco_err_exit() */
 
-int
-nco_typ_lng(nc_type nco_typ) 
+size_t
+nco_typ_lng /* [fnc]   */
+(nc_type nco_typ) 
 { 
   /* Purpose: Return native size of specified netCDF type
      Routine is used to determine memory required to store variables in RAM */
@@ -101,7 +70,7 @@ nco_typ_lng(nc_type nco_typ)
     return sizeof(unsigned char); 
   case NC_BYTE: 
     return sizeof(signed char); 
-  default: nco_dfl_case_nctype_err(); break;
+  default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */ 
   
   /* Some C compilers, e.g., SGI cc, need a return statement at the end of non-void functions */
@@ -109,7 +78,8 @@ nco_typ_lng(nc_type nco_typ)
 } /* end nco_typ_lng() */ 
 
 char *
-nco_typ_sng(nc_type type)
+nco_typ_sng /* [fnc]   */
+(nc_type type)
 /*  
    nc_type type: I [enm] netCDF type
    char *nco_typ_sng(): O [sng] string describing type
@@ -128,19 +98,16 @@ nco_typ_sng(nc_type type)
     return "NC_CHAR";
   case NC_BYTE:
     return "NC_BYTE";
-  default: nco_dfl_case_nctype_err(); break;
+  default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
 
   /* Some C compilers, e.g., SGI cc, need a return statement at the end of non-void functions */
   return (char *)NULL;
 } /* end nco_typ_sng() */
 
-char *
-c_type_nm(nc_type type)
-/*  
-   nc_type type: I netCDF type
-   char *c_type_nm(): O string describing type
-*/
+char * /* O [sng] Native C type */
+c_typ_nm /* [fnc] Return string describing native C type */
+(nc_type type) /* O [enm] netCDF type */
 {
   /* Purpose: Divine internal (native) C type string from netCDF external type enum
      fxm: This breaks on Crays where both NC_FLOAT and NC_DOUBLE are native type float */
@@ -150,51 +117,72 @@ c_type_nm(nc_type type)
   case NC_DOUBLE:
     return "double";
   case NC_INT:
-    return "long";
+    return "signed long int";
   case NC_SHORT:
-    return "short";
+    return "signed short int";
   case NC_CHAR:
     return "unsigned char";
   case NC_BYTE:
     return "signed char";
-  default: nco_dfl_case_nctype_err(); break;
+  default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
 
   /* Some C compilers, e.g., SGI cc, need a return statement at the end of non-void functions */
   return (char *)NULL;
-} /* end c_type_nm() */
+} /* end c_typ_nm() */
 
-char *
-fortran_type_nm(nc_type type)
-/*  
-   nc_type type: I netCDF type
-   char *fortran_type_nm(): O string describing type
-*/
+char * /* O [sng] Native Fortran77 type */
+f77_typ_nm /* [fnc] Return string describing native Fortran77 type */
+(nc_type type) /* O [enm] netCDF type */
 {
-  /* Purpose: Divine internal (native) Fortran type string from netCDF external type enum
-     fxm: This breaks on Crays where both NC_FLOAT and NC_DOUBLE are native type real */
+  /* Purpose: Divine internal (native) Fortran type string from netCDF external type enum */
   switch(type){
   case NC_FLOAT:
-    return "real";
+    return "real*4";
   case NC_DOUBLE:
-    return "double precision";
+    return "real*8";
   case NC_INT:
-    return "integer";
+    return "integer*4";
   case NC_SHORT:
     return "integer*2";
   case NC_CHAR:
     return "character";
   case NC_BYTE:
-    return "char";
-  default: nco_dfl_case_nctype_err(); break;
+    return "character";
+  default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
 
   /* Some C compilers, e.g., SGI cc, need a return statement at the end of non-void functions */
   return (char *)NULL;
-} /* end fortran_type_nm() */
+} /* end f77_typ_nm() */
+
+char * /* O [sng] Native Fortran90 type */
+f90_typ_nm /* [fnc] Return string describing native Fortran90 type */
+(nc_type type) /* O [enm] netCDF type */
+{
+  /* Purpose: Divine internal (native) Fortran type string from netCDF external type enum */
+  switch(type){
+  case NC_FLOAT:
+    return "real(selected_real_kind(p=6))";
+  case NC_DOUBLE:
+    return "real(selected_real_kind(p=12))";
+  case NC_INT:
+    return "integer(selected_int_kind(6))";
+  case NC_SHORT:
+    return "integer(selected_int_kind(2))";
+  case NC_CHAR:
+    return "character(1)";
+  case NC_BYTE:
+    return "character(1)";
+  default: nco_dfl_case_nc_type_err(); break;
+  } /* end switch */
+
+  /* Some C compilers, e.g., SGI cc, need a return statement at the end of non-void functions */
+  return (char *)NULL;
+} /* end f90_typ_nm() */
 
 void 
-nco_dfl_case_nctype_err(void)
+nco_dfl_case_nc_type_err(void) /* [fnc]   */
 {
   /* Purpose: Convenience routine for printing error and exiting when
      switch(nctype) statement receives an illegal default case
@@ -205,25 +193,25 @@ nco_dfl_case_nctype_err(void)
      All these default statements can be removed with netCDF3 interface
      so perhaps these should be surrounded with #ifdef NETCDF2_ONLY
      constructs, but they actually do make sense for netCDF3 as well
-     so I have implemented a uniform error function, nco_dfl_case_nctype_err(), 
+     so I have implemented a uniform error function, nco_dfl_case_nc_type_err(), 
      to be called by all routines which emit errors only when compiled with
      NETCDF2_ONLY.
      This makes the behavior easy to modify or remove in the future.
 
      Placing this in its own routine also has the virtue of saving many lines 
      of code since this function is used in many many switch() statements. */
-  char sbr_nm[]="nco_dfl_case_nctype_err()";
+  char sbr_nm[]="nco_dfl_case_nc_type_err()";
   (void)fprintf(stdout,"%s: ERROR switch(nctype) statement fell through to default case, which is illegal.\nNot handling the default case causes gcc to emit warnings when compiling NCO with the NETCDF2_ONLY token (because nctype definition is braindead in netCDF2.x). Exiting...\n",sbr_nm);
   abort();
   exit(EXIT_FAILURE);
-} /* end nco_dfl_case_nctype_err() */
+} /* end nco_dfl_case_nc_type_err() */
 
 /* Begin file-level routines */
 int
 nco_create(const char *fl_nm,int cmode,int *nc_id)
 {
   /* Purpose: Wrapper for nc_create() */
-  int rcd=NC_NOERR ;
+  int rcd=NC_NOERR;
   int fl_in_typ=nco_fl_typ_nc; /* [enm] File format */
   if(fl_in_typ == nco_fl_typ_nc){
     rcd=nc_create(fl_nm,cmode,nc_id);
@@ -494,21 +482,18 @@ nco_inq_var(int nc_id,int var_id,char *var_nm,nc_type *var_typ,int *dmn_nbr,int
 } /* end nco_inq_var */
 
 int 
-nco_inq_varid(int nc_id,char *var_nm,int *var_id)
+nco_inq_varid(const int nc_id,const char * const var_nm,int * const var_id)
 {
   /* Purpose: Wrapper for nc_inq_varid() */
   int rcd=NC_NOERR;
   rcd=nc_inq_varid(nc_id,var_nm,var_id);
-  if(rcd == NC_ENOTVAR){
-    (void)fprintf(stdout,"ERROR nco_inq_varid() reports requested variable \"%s\" is not in input file\n",var_nm);
-    exit(EXIT_FAILURE);
-  } /* endif */
+  if(rcd == NC_ENOTVAR) (void)fprintf(stdout,"ERROR nco_inq_varid() reports requested variable \"%s\" is not in input file\n",var_nm);
   if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_inq_varid");
   return rcd;
 } /* end nco_inq_varid */
 
 int 
-nco_inq_varid_flg(int nc_id,char *var_nm,int *var_id)
+nco_inq_varid_flg(const int nc_id,const char * const var_nm,int * const var_id)
 {
   /* Purpose: Wrapper for nc_inq_varid_flg() which does not require success */
   int rcd=NC_NOERR;
@@ -603,7 +588,7 @@ nco_get_var1(int nc_id,int var_id,const long *srt,void *vp,nc_type var_typ)
   case NC_SHORT: rcd=nc_get_var1_short(nc_id,var_id,(size_t *)srt,(short *)vp); break;
   case NC_CHAR: rcd=nc_get_var1_text(nc_id,var_id,(size_t *)srt,(char *)vp); break;
   case NC_BYTE: rcd=nc_get_var1_schar(nc_id,var_id,(size_t *)srt,(signed char *)vp); break;
-  default: nco_dfl_case_nctype_err(); break;
+  default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_var1");
   return rcd;
@@ -621,7 +606,7 @@ nco_put_var1(int nc_id,int var_id,const long *srt,void *vp,nc_type type)
   case NC_SHORT: rcd=nc_put_var1_short(nc_id,var_id,(size_t *)srt,(short *)vp); break;
   case NC_CHAR: rcd=nc_put_var1_text(nc_id,var_id,(size_t *)srt,(const char *)vp); break;
   case NC_BYTE: rcd=nc_put_var1_schar(nc_id,var_id,(size_t *)srt,(signed char *)vp); break;
-  default: nco_dfl_case_nctype_err(); break;
+  default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var1");
   return rcd;
@@ -639,7 +624,7 @@ nco_get_vara(int nc_id,int var_id,const long *srt,const long *cnt,void *vp,nc_ty
   case NC_SHORT: rcd=nc_get_vara_short(nc_id,var_id,(size_t *)srt,(size_t *)cnt,(short *)vp); break;
   case NC_CHAR: rcd=nc_get_vara_text(nc_id,var_id,(size_t *)srt,(size_t *)cnt,(char *)vp); break;
   case NC_BYTE: rcd=nc_get_vara_schar(nc_id,var_id,(size_t *)srt,(size_t *)cnt,(signed char *)vp); break;
-  default: nco_dfl_case_nctype_err(); break;
+  default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_vara");
   return rcd;
@@ -657,7 +642,7 @@ nco_put_vara(int nc_id,int var_id,const long *srt,const long *cnt,void *vp,nc_ty
   case NC_SHORT: rcd=nc_put_vara_short(nc_id,var_id,(size_t *)srt,(size_t *)cnt,(short *)vp); break;
   case NC_CHAR: rcd=nc_put_vara_text(nc_id,var_id,(size_t *)srt,(size_t *)cnt,(const char *)vp); break;
   case NC_BYTE: rcd=nc_put_vara_schar(nc_id,var_id,(size_t *)srt,(size_t *)cnt,(signed char *)vp); break;
-  default: nco_dfl_case_nctype_err(); break;
+  default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_vara");
   return rcd;
@@ -676,7 +661,7 @@ nco_get_varm(int nc_id,int var_id,const long *srt,const long *cnt,const long *sr
   case NC_SHORT: rcd=nc_get_varm_short(nc_id,var_id,(size_t *)srt,(size_t *)cnt,(ptrdiff_t *)srd,(ptrdiff_t *)map,(short *)vp); break;
   case NC_CHAR: rcd=nc_get_varm_text(nc_id,var_id,(size_t *)srt,(size_t *)cnt,(ptrdiff_t *)srd,(ptrdiff_t *)map,(char *)vp); break;
   case NC_BYTE: rcd=nc_get_varm_schar(nc_id,var_id,(size_t *)srt,(size_t *)cnt,(ptrdiff_t *)srd,(ptrdiff_t *)map,(signed char *)vp); break;
-  default: nco_dfl_case_nctype_err(); break;
+  default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_varm");
   return rcd;
@@ -694,7 +679,7 @@ nco_put_varm(int nc_id,int var_id,const long *srt,const long *cnt,const long *sr
   case NC_SHORT: rcd=nc_put_varm_short(nc_id,var_id,(size_t *)srt,(size_t *)cnt,(ptrdiff_t *)srd,(ptrdiff_t *)map,(short *)vp); break;
   case NC_CHAR: rcd=nc_put_varm_text(nc_id,var_id,(size_t *)srt,(size_t *)cnt,(ptrdiff_t *)srd,(ptrdiff_t *)map,(const char *)vp); break;
   case NC_BYTE: rcd=nc_put_varm_schar(nc_id,var_id,(size_t *)srt,(size_t *)cnt,(ptrdiff_t *)srd,(ptrdiff_t *)map,(signed char *)vp); break;
-  default: nco_dfl_case_nctype_err(); break;
+  default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_varm");
   return rcd;
@@ -748,7 +733,7 @@ nco_inq_attid_flg(int nc_id,int var_id,const char *att_nm,int *att_id)
   if(rcd == NC_ENOTATT) return rcd;
   if(rcd != NC_NOERR){
     (void)fprintf(stderr,"var_id: %d, att_nm: %s\n",var_id,att_nm);
-    nco_err_exit(rcd,"nco_inq_att_flg");
+    nco_err_exit(rcd,"nco_inq_attid_flg");
   } /* endif */
   return rcd;
 } /* end nco_inq_attid_flg */
@@ -825,7 +810,7 @@ nco_put_att(int nc_id,int var_id,const char *att_nm,nc_type att_typ,long att_len
   case NC_SHORT: rcd=nc_put_att_short(nc_id,var_id,att_nm,att_typ,(size_t)att_len,(short *)vp); break;
   case NC_CHAR: rcd=nc_put_att_text(nc_id,var_id,att_nm,(size_t)att_len,(const char *)vp); break;
   case NC_BYTE: rcd=nc_put_att_schar(nc_id,var_id,att_nm,att_typ,(size_t) att_len,(signed char *)vp); break;
-  default: nco_dfl_case_nctype_err(); break;
+  default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_att");
   return rcd;
@@ -843,7 +828,7 @@ nco_get_att(int nc_id,int var_id,const char *att_nm,void *vp,nc_type att_typ)
   case NC_SHORT: rcd=nc_get_att_short(nc_id,var_id,att_nm,(short *)vp); break;
   case NC_CHAR: rcd=nc_get_att_text(nc_id,var_id,att_nm,(char *)vp); break;
   case NC_BYTE: rcd=nc_get_att_schar(nc_id,var_id,att_nm,(signed char *)vp); break;
-  default: nco_dfl_case_nctype_err(); break;
+  default: nco_dfl_case_nc_type_err(); break;
   } /* end switch */
   if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_att");
   return rcd;
