@@ -1,10 +1,10 @@
-/* $Header: /cvsroot/nco/nco/src/nco/ncrename.c,v 1.31 2001/12/29 05:52:50 zender Exp $ */
+/* $Header: /cvsroot/nco/nco/src/nco/ncrename.c,v 1.52 2003/01/10 18:05:14 rorik Exp $ */
 
 /* ncrename -- netCDF renaming operator */
 
 /* Purpose: Rename dimensions, variables, and attributes of a netCDF file */
 
-/* Copyright (C) 1995--2002 Charlie Zender
+/* Copyright (C) 1995--2003 Charlie Zender
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -44,6 +44,10 @@
    ncrename -d lon,new_lon -v scalar_var,new_scalar_var -a long_name,new_long_name in.nc foo.nc
    */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h> /* Autotools tokens */
+#endif /* !HAVE_CONFIG_H */
+
 /* Standard C headers */
 #include <math.h> /* sin cos cos sin 3.14159 */
 #include <stdio.h> /* stderr, FILE, NULL, etc. */
@@ -52,9 +56,11 @@
 #include <sys/stat.h> /* stat() */
 #include <time.h> /* machine time */
 #include <unistd.h> /* all sorts of POSIX stuff */
-/* #include <assert.h> */ /* assert() debugging macro */
-/* #include <errno.h> */ /* errno */
-/* #include <malloc.h> */ /* malloc() stuff */
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>  /* getopt_long() */
+#else
+#include "nco_getopt.h"
+#endif /* HAVE_GETOPT_H */
 
 /* 3rd party vendors */
 #include <netcdf.h> /* netCDF definitions */
@@ -62,18 +68,11 @@
 
 /* #define MAIN_PROGRAM_FILE MUST precede #include nco.h */
 #define MAIN_PROGRAM_FILE
-#include "nco.h" /* NCO definitions */
-
-/*#include <curses.h>*/
+#include "libnco.h" /* netCDF operator library */
 
 int 
 main(int argc,char **argv)
 {
-/*  int initscr(void);*/
-/*  int cbreak(void);*/
-
-  rnm_sct *prs_rnm_lst(int,char **);
-  int prs_att(rnm_sct *, char *);
   bool OUTPUT_TO_NEW_NETCDF_FILE=False;
   bool FORCE_APPEND=False; /* Option A */
   bool FORCE_OVERWRITE=False; /* Option O */
@@ -93,8 +92,8 @@ main(int argc,char **argv)
   char *fl_pth=NULL; /* Option p */
   char *time_bfr_srt;
   char *cmd_ln;
-  char CVS_Id[]="$Id: ncrename.c,v 1.31 2001/12/29 05:52:50 zender Exp $"; 
-  char CVS_Revision[]="$Revision: 1.31 $";
+  char CVS_Id[]="$Id: ncrename.c,v 1.52 2003/01/10 18:05:14 rorik Exp $"; 
+  char CVS_Revision[]="$Revision: 1.52 $";
   
   extern char *optarg;
   
@@ -116,8 +115,29 @@ main(int argc,char **argv)
 
   time_t clock;
 
-  /* Start the clock and save the command line */ 
-  cmd_ln=cmd_ln_sng(argc,argv);
+  static struct option opt_lng[] =
+    {
+      {"attribute",required_argument,0,'a'},
+      {"debug",required_argument,0,'D'},
+      {"dbg_lvl",required_argument,0,'D'},
+      {"dimension",required_argument,0,'d'},
+      {"dmn",required_argument,0,'d'},
+      {"history",no_argument,0,'h'},
+      {"hst",no_argument,0,'h'},
+      {"local",required_argument,0,'l'},
+      {"overwrite",no_argument,0,'O'},
+      {"path",required_argument,0,'p'},
+      {"revision",no_argument,0,'r'},
+      {"variable",required_argument,0,'v'},
+      {"version",no_argument,0,'r'},
+      {"vrs",no_argument,0,'r'},
+      {"help",no_argument,0,'?'},
+      {0,0,0,0}
+    };
+  int opt_idx=0;  /* Index of current long option into opt_lng array */
+
+  /* Start clock and save command line */ 
+  cmd_ln=nco_cmd_ln_sng(argc,argv);
   clock=time((time_t *)NULL);
   time_bfr_srt=ctime(&clock); time_bfr_srt=time_bfr_srt; /* Avoid compiler warning until variable is used for something */
   
@@ -125,8 +145,8 @@ main(int argc,char **argv)
   prg_nm=prg_prs(argv[0],&prg);
 
   /* Parse command line arguments */
-  opt_sng="a:D:d:hl:Op:rv:";
-  while((opt = getopt(argc,argv,opt_sng)) != EOF){
+  opt_sng="a:D:d:hl:Op:rv:-:";
+  while((opt = getopt_long(argc,argv,opt_sng,opt_lng,&opt_idx)) != EOF){
     switch(opt){
     case 'A': /* Toggle FORCE_APPEND */
       FORCE_APPEND=!FORCE_APPEND;
@@ -160,39 +180,47 @@ main(int argc,char **argv)
     case 'r': /* Print CVS program information and copyright notice */
       (void)copyright_prn(CVS_Id,CVS_Revision);
       (void)nco_lib_vrs_prn();
-      exit(EXIT_SUCCESS);
+      nco_exit(EXIT_SUCCESS);
       break;
     case 'v': /* Copy argument for later processing */
       var_rnm_arg[nbr_var_rnm]=optarg;
       nbr_var_rnm++;
       break;
+    case '?': /* Print proper usage */
+      (void)nco_usg_prn();
+      nco_exit(EXIT_FAILURE);
+      break;
+    case '-': /* Long options are not allowed */
+      (void)fprintf(stderr,"%s: ERROR Long options are not available in this build. Use single letter options instead.\n",prg_nm_get());
+      nco_exit(EXIT_FAILURE);
+      break;
     default: /* Print proper usage */
-      (void)usg_prn();
-      exit(EXIT_FAILURE);
+      (void)nco_usg_prn();
+      nco_exit(EXIT_FAILURE);
     } /* end switch */
   } /* end while loop */
   
   /* Process positional arguments and fill in filenames */
-  fl_lst_in=fl_lst_mk(argv,argc,optind,&nbr_fl,&fl_out);
+  fl_lst_in=nco_fl_lst_mk(argv,argc,optind,&nbr_fl,&fl_out);
   if(fl_out != NULL) OUTPUT_TO_NEW_NETCDF_FILE=True; else fl_out=fl_lst_in[0];
 
   if(nbr_var_rnm == 0 && nbr_att_rnm == 0 && nbr_dmn_rnm == 0){
     (void)fprintf(stdout,"%s: ERROR must specify something to rename\n",prg_nm);
-    usg_prn();
-    exit(EXIT_FAILURE);
+    nco_usg_prn();
+    nco_exit(EXIT_FAILURE);
   } /* end if */ 
 
-  /* Make a uniform list of the user-specified rename structures */
-  if(nbr_var_rnm > 0) var_rnm_lst=prs_rnm_lst(nbr_var_rnm,var_rnm_arg);
-  if(nbr_dmn_rnm > 0) dmn_rnm_lst=prs_rnm_lst(nbr_dmn_rnm,dmn_rnm_arg);
-  if(nbr_att_rnm > 0) att_rnm_lst=prs_rnm_lst(nbr_att_rnm,att_rnm_arg);
+  /* Make uniform list of user-specified rename structures */
+  if(nbr_var_rnm > 0) var_rnm_lst=nco_prs_rnm_lst(nbr_var_rnm,var_rnm_arg);
+  if(nbr_dmn_rnm > 0) dmn_rnm_lst=nco_prs_rnm_lst(nbr_dmn_rnm,dmn_rnm_arg);
+  if(nbr_att_rnm > 0) att_rnm_lst=nco_prs_rnm_lst(nbr_att_rnm,att_rnm_arg);
 
-  /* We now have the final list of variables, dimensions, and attributes to rename. */
+  /* We have final list of variables, dimensions, and attributes to rename */
   
   /* Parse filename */
-  fl_in=fl_nm_prs(fl_in,0,&nbr_fl,fl_lst_in,nbr_abb_arg,fl_lst_abb,fl_pth);
+  fl_in=nco_fl_nm_prs(fl_in,0,&nbr_fl,fl_lst_in,nbr_abb_arg,fl_lst_abb,fl_pth);
   /* Make sure file is on local system and is readable or die trying */
-  fl_in=fl_mk_lcl(fl_in,fl_pth_lcl,&FILE_RETRIEVED_FROM_REMOTE_LOCATION);
+  fl_in=nco_fl_mk_lcl(fl_in,fl_pth_lcl,&FILE_RETRIEVED_FROM_REMOTE_LOCATION);
 
   if(OUTPUT_TO_NEW_NETCDF_FILE){
 
@@ -203,7 +231,7 @@ main(int argc,char **argv)
       
       rcd=stat(fl_out,&stat_sct);
 
-      /* If file already exists, then query the user whether to overwrite */
+      /* If file already exists, then query user whether to overwrite */
       if(rcd != -1){
         char usr_reply='z';
 	short nbr_itr=0;
@@ -212,7 +240,7 @@ main(int argc,char **argv)
 	  nbr_itr++;
 	  if(nbr_itr > 10){
 	    (void)fprintf(stdout,"\n%s: ERROR %hd failed attempts to obtain valid interactive input. Assuming non-interactive shell and exiting.\n",prg_nm_get(),nbr_itr-1);
-	    exit(EXIT_FAILURE);
+	    nco_exit(EXIT_FAILURE);
 	  } /* end if */
           (void)fprintf(stdout,"ncrename: overwrite %s (y/n)? ",fl_out);
 	  (void)fflush(stdout);
@@ -223,15 +251,15 @@ main(int argc,char **argv)
         } /* end while */
         
         if(usr_reply == 'n'){
-          exit(EXIT_SUCCESS);
+          nco_exit(EXIT_SUCCESS);
         } /* end if */
       } /* end if */
     } /* end if */
     
-    /* Copy input file to output file and then search through
-       the output, changing names as you go. This avoids the possible XDR translation
+    /* Copy input file to output file and then search through output, 
+       changing names on the fly. This avoids possible XDR translation
        performance penalty of copying each variable with netCDF. */
-    (void)fl_cp(fl_in,fl_out);
+    (void)nco_fl_cp(fl_in,fl_out);
 
   } /* end if */
   
@@ -239,7 +267,7 @@ main(int argc,char **argv)
   rcd=nco_open(fl_out,NC_WRITE,&nc_id);
   (void)nco_redef(nc_id);
 
-  /* Without further ado, change the names */
+  /* Without further ado, change names */
   for(idx=0;idx<nbr_var_rnm;idx++){
     if(var_rnm_lst[idx].old_nm[0] == '.'){
       rcd=nco_inq_varid_flg(nc_id,var_rnm_lst[idx].old_nm+1,&var_rnm_lst[idx].id);
@@ -249,8 +277,7 @@ main(int argc,char **argv)
       }else{
 	(void)fprintf(stderr,"%s: WARNING Variable \"%s\" not present in %s, skipping it.\n",prg_nm,var_rnm_lst[idx].old_nm+1,fl_in);
       } /* end if */
-      
-    }else{
+    }else{ /* Variable name does not contain '.' so variable presence is required */
       rcd=nco_inq_varid(nc_id,var_rnm_lst[idx].old_nm,&var_rnm_lst[idx].id);
       (void)nco_rename_var(nc_id,var_rnm_lst[idx].id,var_rnm_lst[idx].new_nm);
       if(dbg_lvl > 0) (void)fprintf(stderr,"Renamed variable \"%s\" to \"%s\"\n",var_rnm_lst[idx].old_nm,var_rnm_lst[idx].new_nm);
@@ -285,45 +312,54 @@ main(int argc,char **argv)
       int var_id;
       int nbr_rnm=0;
       
-      /* Check if we are renaming attribute of a single variable */
-      if(strchr(att_rnm_lst[idx].old_nm,':')){
+      /* Rename attribute of single variable... */
+      if(strchr(att_rnm_lst[idx].old_nm,'@')){
 	/* Get variable name from old name */
 	char var_nm[NC_MAX_NAME];
-	if(prs_att((att_rnm_lst+idx),var_nm)){
+	if(nco_prs_att((att_rnm_lst+idx),var_nm)){
 	  /* Get var_id of variable */
-	  rcd=nco_inq_varid_flg(nc_id,var_nm,&var_id);
+	  if(var_nm[0] == '.'){
+	    rcd=nco_inq_varid_flg(nc_id,var_nm+1,&var_id);
+	    if(rcd != NC_NOERR){
+	      (void)fprintf(stderr,"%s: WARNING Variable \"%s\" not present in %s, skipping it.\n",prg_nm,var_nm+1,fl_in);
+	      continue;
+	    } /* end if */
+	  }else{ /* Variable name does not contain '.' so variable presence is required */
+	    rcd=nco_inq_varid(nc_id,var_nm,&var_id);
+	  } /* end if */
 	  if(rcd == NC_NOERR){
 	    if(att_rnm_lst[idx].old_nm[0] == '.'){
 	      /* Preceding '.' means attribute may not be present */
-	      rcd=nco_inq_attid(nc_id,var_id,att_rnm_lst[idx].old_nm+1,&att_rnm_lst[idx].id);	  
+	      rcd=nco_inq_attid_flg(nc_id,var_id,att_rnm_lst[idx].old_nm+1,&att_rnm_lst[idx].id);	  
 	      if(rcd == NC_NOERR){
 		(void)nco_rename_att(nc_id,var_id,att_rnm_lst[idx].old_nm+1,att_rnm_lst[idx].new_nm);
 		nbr_rnm++;
-		if(dbg_lvl > 0) (void)fprintf(stderr,"Renamed attribute \"%s\" to \"%s\" for variable \"%s\"\n",att_rnm_lst[idx].old_nm+1,att_rnm_lst[idx].new_nm,var_nm);
-	      } /* endif */
+		if(dbg_lvl > 0) (void)fprintf(stderr,"Renamed attribute \"%s\" to \"%s\" for variable \"%s\"\n",att_rnm_lst[idx].old_nm+1,att_rnm_lst[idx].new_nm,(var_nm[0] == '.' ? var_nm+1 : var_nm));
+	      }else{
+		(void)fprintf(stderr,"%s: WARNING Attribute \"%s\" not present in variable \"%s\"\n",prg_nm,att_rnm_lst[idx].old_nm+1,(var_nm[0] == '.' ? var_nm+1 : var_nm));
+	      } /* endelse */
 	    }else{ 
+	      (void)fprintf(stderr,"DEBUGGING TODO 240. fxm 1: %d, %s\n",var_id,att_rnm_lst[idx].old_nm);
 	      rcd=nco_inq_attid(nc_id,var_id,att_rnm_lst[idx].old_nm,&att_rnm_lst[idx].id);
 	      if(rcd == NC_NOERR){
 		(void)nco_rename_att(nc_id,var_id,att_rnm_lst[idx].old_nm,att_rnm_lst[idx].new_nm);
 		nbr_rnm++;
-		if(dbg_lvl > 0) (void)fprintf(stderr,"Renamed attribute \"%s\" to \"%s\" for variable \"%s\"\n",att_rnm_lst[idx].old_nm,att_rnm_lst[idx].new_nm,var_nm);
+		if(dbg_lvl > 0) (void)fprintf(stderr,"Renamed attribute \"%s\" to \"%s\" for variable \"%s\"\n",att_rnm_lst[idx].old_nm,att_rnm_lst[idx].new_nm,(var_nm[0] == '.' ? var_nm+1 : var_nm));
 	      } /* endif attribute is present */
 	    } /* endelse attribute must be present */  
 	  }else{
 	    (void)fprintf(stderr,"%s: WARNING variable \"%s\" not present in %s\n",prg_nm,var_nm,fl_in);
 	  } /* endelse variable is present */
-	} /*  */
-      }else{ /* endif */
-	for(var_id=-1;var_id<nbr_var_fl;var_id++){
+	} /* end if renaming single variable */
+      }else{ /* ...or rename attribute for all variables... */
+	for(var_id=-1;var_id<nbr_var_fl;var_id++){ /* Start loop at -1 for global attributes */
 	  if(att_rnm_lst[idx].old_nm[0] == '.'){
-	    
-	    rcd=nco_inq_attid(nc_id,var_id,att_rnm_lst[idx].old_nm+1,&att_rnm_lst[idx].id);
+	    /* Rename attribute if variable contains attribute else do nothing */
+	    rcd=nco_inq_attid_flg(nc_id,var_id,att_rnm_lst[idx].old_nm+1,&att_rnm_lst[idx].id);
 	    if(rcd == NC_NOERR){
 	      (void)nco_rename_att(nc_id,var_id,att_rnm_lst[idx].old_nm+1,att_rnm_lst[idx].new_nm);
 	      nbr_rnm++;
-	      
-	      /* There can be many attributes of the same name in a given file
-		 Inform user what has actually been renamed */
+	      /* Inform user which variable had attribute renamed */
 	      if(var_id > -1){
 		char var_nm[NC_MAX_NAME];
 		
@@ -333,14 +369,15 @@ main(int argc,char **argv)
 		if(dbg_lvl > 0) (void)fprintf(stderr,"Renamed global attribute \"%s\" to \"%s\"\n",att_rnm_lst[idx].old_nm+1,att_rnm_lst[idx].new_nm);
 	      } /* end else */
 	    } /* end if */
+	    
 	  }else{
-	    rcd=nco_inq_attid(nc_id,var_id,att_rnm_lst[idx].old_nm,&att_rnm_lst[idx].id);
+	    /* Rename attribute or die trying */
+/*	    (void)fprintf(stderr,"DEBUGGING TODO 240. fxm 2: %d, %s\n",var_id,att_rnm_lst[idx].old_nm); */
+	    rcd=nco_inq_attid_flg(nc_id,var_id,att_rnm_lst[idx].old_nm,&att_rnm_lst[idx].id);
 	    if(rcd == NC_NOERR){
 	      (void)nco_rename_att(nc_id,var_id,att_rnm_lst[idx].old_nm,att_rnm_lst[idx].new_nm);
 	      nbr_rnm++;
-	      
-	      /* There can be many attributes of the same name in a file, so
-		 tell the user what was actually changed */
+	      /* Inform user which variable had attribute renamed */
 	      if(var_id > -1){
 		char var_nm[NC_MAX_NAME];
 		
@@ -352,23 +389,23 @@ main(int argc,char **argv)
 	    } /* end if */
 	  } /* end else */
 	} /* end loop over var_id */
-      } /* end else */
+      } /* end if renaming attribute for all variables */
       /* See to it that any mandatory renaming was performed, else abort */
       if(nbr_rnm == 0){
 	if(att_rnm_lst[idx].old_nm[0] == '.'){
-	  (void)fprintf(stderr,"%s: WARNING Attribute \"%s\" not present in %s, skipping it.\n",prg_nm,att_rnm_lst[idx].old_nm+1,fl_in);
+	  (void)fprintf(stderr,"%s: WARNING Attribute \"%s\" not renamed because not found in searched variable(s)\n",prg_nm,att_rnm_lst[idx].old_nm+1);
 	}else{
 	  (void)fprintf(stdout,"%s: ERROR Attribute \"%s\" not present in %s, aborting.\n",prg_nm,att_rnm_lst[idx].old_nm,fl_in);
-	  exit(EXIT_FAILURE);
+	  nco_exit(EXIT_FAILURE);
 	} /* end else */
       } /* end if */
       
-    } /* end loop over idx */
+    } /* end loop over attributes to rename */
      
-  }/* end if */
+  } /* end if renaming attributes */
   
-  /* Catenate the timestamped command line to the "history" global attribute */
-  if(HISTORY_APPEND) (void)hst_att_cat(nc_id,cmd_ln);
+  /* Catenate timestamped command line to "history" global attribute */
+  if(HISTORY_APPEND) (void)nco_hst_att_cat(nc_id,cmd_ln);
   
   /* Take file out of define mode */
   (void)nco_enddef(nc_id);
@@ -377,95 +414,9 @@ main(int argc,char **argv)
   nco_close(nc_id);
   
   /* Remove local copy of file */
-  if(FILE_RETRIEVED_FROM_REMOTE_LOCATION && REMOVE_REMOTE_FILES_AFTER_PROCESSING) (void)fl_rm(fl_in);
+  if(FILE_RETRIEVED_FROM_REMOTE_LOCATION && REMOVE_REMOTE_FILES_AFTER_PROCESSING) (void)nco_fl_rm(fl_in);
 
-  Exit_gracefully();
+  nco_exit_gracefully();
   return EXIT_SUCCESS;
 } /* end main() */
 
-rnm_sct *
-prs_rnm_lst(int nbr_rnm,char **rnm_arg)
-{
-  /* Routine to set old_nm, new_nm elements of a rename structure
-     This routine merely fills rename structure and does not attempt to validate 
-     presence of variables in input netCDF file. */
-
-  rnm_sct *rnm_lst;
-
-  int idx;
-  int len_arg_1;
-  int len_arg_2;
-
-  rnm_lst=(rnm_sct *)nco_malloc(nbr_rnm*sizeof(rnm_sct));
-
-  for(idx=0;idx<nbr_rnm;idx++){
-    char *comma_1_cp;
-
-    /* Find the positions of the commas and number of characters
-       between (non-inclusive) them */
-    comma_1_cp=strchr(rnm_arg[idx],',');
-    
-    /* Before doing any pointer arithmetic, make sure the pointers 
-       are valid. */
-    if(comma_1_cp == NULL){
-      (void)usg_prn();
-      exit(EXIT_FAILURE);
-    } /* end if */
-    
-    len_arg_1=comma_1_cp-rnm_arg[idx]; 
-    len_arg_2=rnm_arg[idx]+strlen(rnm_arg[idx])-comma_1_cp-1; 
-    
-    /* If the length of either of the arguments is zero then exit */
-    if(len_arg_1 <= 0 || len_arg_2 <= 0){
-      (void)usg_prn();
-      exit(EXIT_FAILURE);
-    } /* end if */
-    
-    /* Assign the pointers to the member of the rnm_lst */
-    rnm_lst[idx].old_nm=rnm_arg[idx];
-    rnm_lst[idx].new_nm=comma_1_cp+1;
-
-    /* Manually null-terminate the arguments */
-    rnm_lst[idx].old_nm[len_arg_1]='\0';
-    rnm_lst[idx].new_nm[len_arg_2]='\0';
-    
-  } /* End loop over rnm_lst */
-
-  if(dbg_lvl == 5){
-    for(idx=0;idx<nbr_rnm;idx++){
-      (void)fprintf(stderr,"%s\n",rnm_lst[idx].old_nm);
-      (void)fprintf(stderr,"%s\n",rnm_lst[idx].new_nm);
-    } /* end loop over idx */
-  } /* end debug */
-
-  return rnm_lst;
-} /* end prs_rnm_lst() */
-
-int
-prs_att(rnm_sct *rnm_att,char *var_nm)
-{
-  /* Purpose: Check if attribute name space contains variable name before attribute name of the form  var_nm:att_name
-     Attribute name is then extracted from from old_nm and new_nm as necessary */
-  
-  char *colon;
-  int len;
-  colon = strchr(rnm_att->old_nm,':');	
-  if( colon == NULL ) return 0;
-  
-  len = strlen(rnm_att->old_nm);
-  
-  if( len < 3 || colon == rnm_att->old_nm || colon == rnm_att->old_nm + len -1 ) return 0;
-  *colon = '\0';
-  /* now copy just variable name */
-  strcpy(var_nm,rnm_att->old_nm);
-  /* now set to just attribute name */
-  rnm_att->old_nm = colon +1 ; 
-    
-  colon = strchr(rnm_att->new_nm,':');	
-  if( colon  ) {
-    len = strlen(rnm_att->new_nm);
-    if ( (colon - rnm_att->new_nm) < len ) rnm_att->new_nm = colon +1;
-    else return 0;
-  } 
-  return 1;
-} /* end prs_att() */
