@@ -1,8 +1,10 @@
-// $Header: /cvsroot/nco/nco/src/nco_c++/nco_att.cc,v 1.1 2002/01/11 23:20:29 zender Exp $ 
+// $Header: /cvsroot/nco/nco/src/nco_c++/nco_att.cc,v 1.4 2003/01/16 09:26:55 zender Exp $ 
 
 // Implementation (declaration) of C++ interface to netCDF attribute routines
 
-/* */
+/* Copyright (C) 2001--2003 Charlie Zender
+   This software is distributed under the terms of the GNU General Public License
+   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include <nco_att.hh> // C++ interface to netCDF attribute routines
 
@@ -53,7 +55,7 @@ nco_put_att // [fnc] Create attribute
   // Purpose: Wrapper for nco_put_att<float>()
   const size_t att_sz(1);
   //  int rcd=nco_put_att(nc_id,var_id,att_nm,att_sz,&(const_cast<float &>(att_val)),att_typ);
-  float foo_val(att_val);
+  // float foo_val(att_val);
   // int rcd=nco_put_att(nc_id,var_id,att_nm,att_sz,&foo_val,att_typ);
   int rcd=nc_put_att_float(nc_id,var_id,att_nm.c_str(),att_typ,att_sz,&(const_cast<float &>(att_val)));
   return rcd;
