@@ -1,8 +1,10 @@
-// $Header: /cvsroot/nco/nco/src/nco_c++/nco_utl.cc,v 1.1 2002/01/11 23:20:29 zender Exp $ 
+// $Header: /cvsroot/nco/nco/src/nco_c++/nco_utl.cc,v 1.7 2003/01/16 09:26:55 zender Exp $ 
 
 // Implementation (declaration) of C++ interface utilities for netCDF routines
 
-/* */
+/* Copyright (C) 2001--2003 Charlie Zender
+   This software is distributed under the terms of the GNU General Public License
+   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include <nco_utl.hh> // C++ interface utilities for netCDF routines
 
@@ -44,7 +46,11 @@ nco_err_exit // [fnc] Lookup, print netCDF error message, exit
 #ifdef ABORT_ON_ERROR
     std::abort(); // [fnc] Produce core dump
 #else
+#ifndef SGIMP64 // fxm: SGI IRIX CC does not support std::exit nor have cstdlib
     std::exit(EXIT_FAILURE); // [fnc] Exit nicely
+#else // SGIMP64
+    exit(EXIT_FAILURE); // [fnc] Exit nicely
+#endif // SGIMP64
 #endif // !ABORT_ON_ERROR
   } // endif error
 } // end nco_err_exit()
@@ -70,7 +76,7 @@ nco_inq_varsz // [fnc] Compute size of variable
     rcd=nco_inq_dimlen(nc_id,dmn_id[dmn_idx],dmn_sz);
     var_sz*=dmn_sz; // [nbr] Variable size
   } // end loop over dmn
-  delete[]dmn_id; // [id] Dimension IDs
+  delete []dmn_id; // [id] Dimension IDs
   return rcd;
 } // end nco_inq_varsz()
 
@@ -221,7 +227,16 @@ nco_dfl_case_nctype_err(void) // [fnc] Handle illegal nc_type references
      of code since this function is used in many many switch() statements. */
   const std::string sbr_nm("nco_dfl_case_nctype_err()");
   std::cout << sbr_nm << ": ERROR switch(nctype) statement fell through to default case, which is illegal.\nNot handling the default case causes gcc to emit warnings when compiling NCO with the NETCDF2_ONLY token (because nctype defintion is braindead in netCDF2). Exiting..." << std::endl;
-  std::exit(EXIT_FAILURE);
+#ifdef ABORT_ON_ERROR
+  std::abort(); // [fnc] Produce core dump
+#else
+#ifndef SGIMP64 // fxm: SGI IRIX CC does not support std::exit nor have cstdlib
+    std::exit(EXIT_FAILURE); // [fnc] Exit nicely
+#else // SGIMP64
+    exit(EXIT_FAILURE); // [fnc] Exit nicely
+#endif // SGIMP64
+#endif // !ABORT_ON_ERROR
+
 } // end nco_dfl_case_nctype_err()
 
 // Begin nco_get_xtype() overloads
