@@ -1,8 +1,10 @@
-// $Header: /cvsroot/nco/nco/src/nco_c++/nco_var.cc,v 1.1 2002/01/11 23:20:29 zender Exp $ 
+// $Header: /cvsroot/nco/nco/src/nco_c++/nco_var.cc,v 1.7 2003/01/16 09:26:55 zender Exp $ 
 
-// Implementation (declaration) of C++ interface to netCDF variable routines
+// Purpose: Implementation (declaration) of C++ interface to netCDF variable routines
 
-/* */
+/* Copyright (C) 2001--2003 Charlie Zender
+   This software is distributed under the terms of the GNU General Public License
+   See http://www.gnu.ai.mit.edu/copyleft/gpl.html for full license text */
 
 #include <nco_var.hh> // C++ interface to netCDF variable routines
 
@@ -300,6 +302,24 @@ nco_put_var // [fnc] Write variable to netCDF file
   return rcd;
 } // end nco_put_var<double *>()
 
+int // O [enm] Return success code
+nco_put_var // [fnc] Write variable to netCDF file
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const long double * const &var_val) // I [frc] Variable value
+{
+  // Purpose: Wrapper for nc_put_var_double()
+  const size_t var_sz(nco_inq_varsz(nc_id,var_id)); // [nbr] Number of elements
+  double *var_val_dbl=new double[var_sz]; // [frc] Double precision values
+  for(size_t lmn_idx=0;lmn_idx<var_sz;lmn_idx++){
+    var_val_dbl[lmn_idx]=var_val[lmn_idx];
+  } // end loop over idx
+  int rcd=nc_put_var_double(nc_id,var_id,var_val_dbl);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<long double *>");
+  delete []var_val_dbl;
+  return rcd;
+} // end nco_put_var<long double *>()
+
 int // O [enm] Return success code
 nco_put_var // [fnc] Write variable to netCDF file
 (const int &nc_id, // I [enm] netCDF file ID
@@ -324,6 +344,18 @@ nco_put_var // [fnc] Write variable to netCDF file
   return rcd;
 } // end nco_put_var<long *>()
 
+int // O [enm] Return success code
+nco_put_var // [fnc] Write variable to netCDF file
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const short * const &var_val) // I [frc] Variable value
+{
+  // Purpose: Wrapper for nc_put_var_short()
+  int rcd=nc_put_var_short(nc_id,var_id,var_val);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<short *>");
+  return rcd;
+} // end nco_put_var<short *>()
+
 // Overload 3: Write scalar given ID
 int // O [enm] Return success code
 nco_put_var // [fnc] Write variable to netCDF file
@@ -352,6 +384,20 @@ nco_put_var // [fnc] Write variable to netCDF file
   return rcd;
 } // end nco_put_var<double>()
 
+int // O [enm] Return success code
+nco_put_var // [fnc] Write variable to netCDF file
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const long double &var_val) // I [frc] Variable value
+{
+  // Purpose: Wrapper for nc_put_var1_double()
+  const double var_val_dbl(var_val); // [frc] Double precision value
+  std::valarray<size_t> srt(static_cast<size_t>(0),static_cast<size_t>(nco_inq_varndims(nc_id,var_id)));
+  int rcd=nc_put_var1_double(nc_id,var_id,&srt[0],&(const_cast<double &>(var_val_dbl)));
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<long double>");
+  return rcd;
+} // end nco_put_var<long double>()
+
 int // O [enm] Return success code
 nco_put_var // [fnc] Write variable to netCDF file
 (const int &nc_id, // I [enm] netCDF file ID
@@ -379,7 +425,9 @@ nco_put_var // [fnc] Write variable to netCDF file
   const int dmn_nbr(nco_inq_varndims(nc_id,var_id));
   int rcd; // O [enm] Return success code
   if(dmn_nbr == 0){
-    rcd=nc_put_var1_long(nc_id,var_id,NULL,&(const_cast<long &>(var_val)));
+    size_t srt_dmn_0=0; // Faster than initializing a valarray
+    const size_t *srt_dmn_0_ptr(&srt_dmn_0);
+    rcd=nc_put_var1_long(nc_id,var_id,srt_dmn_0_ptr,&(const_cast<long &>(var_val)));
   }else{
     // Default starting index is 0
     std::valarray<size_t> srt(static_cast<size_t>(0),static_cast<size_t>(dmn_nbr));
@@ -389,6 +437,19 @@ nco_put_var // [fnc] Write variable to netCDF file
   return rcd;
 } // end nco_put_var<long>()
 
+int // O [enm] Return success code
+nco_put_var // [fnc] Write variable to netCDF file
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ const short &var_val) // I [frc] Variable value
+{
+  // Purpose: Wrapper for nc_put_var1_short()
+  std::valarray<size_t> srt(static_cast<size_t>(0),static_cast<size_t>(nco_inq_varndims(nc_id,var_id)));
+  int rcd=nc_put_var1_short(nc_id,var_id,&srt[0],&(const_cast<short &>(var_val)));
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_put_var<short>");
+  return rcd;
+} // end nco_put_var<short>()
+
 // End nco_put_var() overloads
 // Begin nco_get_var() overloads
 
@@ -425,6 +486,27 @@ nco_get_var // [fnc] Ingest variable from netCDF file
   return rcd;
 } // end nco_get_var<double>()
 
+int // O [enm] Return success code
+nco_get_var // [fnc] Ingest variable from netCDF file
+(const int &nc_id, // I [enm] netCDF file ID
+ const int &var_id, // I [id] Variable ID
+ long double *&var_val) // O [frc] Variable value
+{
+  // Purpose: Wrapper for nc_get_var()
+  size_t var_sz; // [nbr] Variable size
+  int rcd=nco_inq_varsz(nc_id,var_id,var_sz);
+  // Allocate space for variable: User is responsible for freeing space
+  var_val=new long double[var_sz]; // [frc] Variable value
+  double *var_val_dbl=new double[var_sz]; // [frc] Double precision values
+  rcd=nc_get_var_double(nc_id,var_id,var_val_dbl);
+  if(rcd != NC_NOERR) nco_err_exit(rcd,"nco_get_var<long double>");
+  for(size_t lmn_idx=0;lmn_idx<var_sz;lmn_idx++){
+    var_val[lmn_idx]=var_val_dbl[lmn_idx];
+  } // end loop over idx
+  delete []var_val_dbl;
+  return rcd;
+} // end nco_get_var<long double>()
+
 int // O [enm] Return success code
 nco_get_var // [fnc] Ingest variable from netCDF file
 (const int &nc_id, // I [enm] netCDF file ID
